@@ -17,7 +17,7 @@ Math::BigNum::Inf - Represents the +Infinity value.
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =head1 SYNOPSIS
 
@@ -1076,16 +1076,13 @@ sub sqr { $_[0]->mul($_[0]) }
 
 =head2 sqrt / isqrt
 
-    $x->sqrt           => Inf | Nan
+    $x->sqrt           => Inf
 
-Square root of C<$x>. Returns Nan when C<$x> is negative.
+Square root of C<$x>.
 
 =cut
 
-sub sqrt {
-    $_[0]->is_neg ? nan() : $_[0]->copy;
-}
-
+*sqrt  = \&inf;
 *isqrt = \&sqrt;
 
 =head2 bsqrt / bisqrt
@@ -1094,19 +1091,18 @@ Square root of C<$x>, changing C<$x> in-place.
 
 =cut
 
-sub bsqrt { $_[0]->is_neg ? $_[0]->bnan : $_[0] }
-
+*bsqrt  = \&binf;
 *bisqrt = \&bsqrt;
 
 =head2 pow / ipow
 
-    $x->pow(BigNum)                # => Inf | Nan | BigNum
-    $x->pow(Scalar)                # => Inf | Nan | BigNum
-    $x->pow(Inf)                   # => Inf | Nan | BigNum
+    $x->pow(BigNum)                # => Inf | BigNum
+    $x->pow(Scalar)                # => Inf | BigNum
+    $x->pow(Inf)                   # => Inf | BigNum
 
     Scalar ** Inf                  # => Inf | BigNum(0)
-    Inf ** BigNum                  # => Inf | Nan | BigNum
-    Inf ** Scalar                  # => Inf | Nan | BigNum
+    Inf ** BigNum                  # => Inf | BigNum
+    Inf ** Scalar                  # => Inf | BigNum
 
 Raises C<$x> to the power C<$y>.
 
@@ -1128,12 +1124,12 @@ multimethod pow => qw(Math::BigNum::Inf Math::BigNum::Nan) => \&nan;
 
 =head2 bpow / bipow
 
-    $x->bpow(BigNum)               # => Inf | Nan | BigNum
-    $x->bpow(Scalar)               # => Inf | Nan | BigNum
-    $x->bpow(Inf)                  # => Inf | Nan | BigNum
+    $x->bpow(BigNum)               # => Inf | BigNum
+    $x->bpow(Scalar)               # => Inf | BigNum
+    $x->bpow(Inf)                  # => Inf | BigNum
 
-    Inf **= BigNum                 # => Inf | Nan | BigNum
-    Inf **= Scalar                 # => Inf | Nan | BigNum
+    Inf **= BigNum                 # => Inf | BigNum
+    Inf **= Scalar                 # => Inf | BigNum
 
 Same C<pow()>, except that it changes C<$x> in-place.
 
@@ -1143,11 +1139,9 @@ multimethod bpow => qw(Math::BigNum::Inf Math::BigNum) => sub {
     my ($x, $y) = @_;
     $y->is_neg      ? $x->bzero
       : $y->is_zero ? $x->bone
-      : $x->is_neg  ? $y->is_int
-          ? $y->is_even
-              ? $x->bneg
-              : $x
-          : $x->bnan
+      : $x->is_neg  ? $y->is_odd
+          ? $x
+          : $x->bneg
       : $x;
 };
 
@@ -1155,11 +1149,9 @@ multimethod bpow => qw(Math::BigNum::Inf $) => sub {
     my ($x, $y) = @_;
     $y < 0         ? $x->bzero
       : $y == 0    ? $x->bone
-      : $x->is_neg ? CORE::int($y) eq $y
-          ? ($y % 2 == 0)
-              ? $x->bneg
-              : $x
-          : $x->bnan()
+      : $x->is_neg ? ($y % 2 != 0)
+          ? $x
+          : $x->bneg
       : $x;
 };
 
@@ -1291,14 +1283,19 @@ Inverse value of +/-Infinity. Always returns zero.
     $x->mod(BigNum)                # => Nan
     $x->mod(Inf)                   # => Nan
 
-    Scalar % Inf                   # => BigNum
+    Scalar % Inf                   # => BigNum | Inf
 
 Returns the remained of C<$x> divided by C<$y>.
 
 =cut
 
+# +x mod +Inf = x
+# +x mod -Inf = -Inf
+# -x mod +Inf = +Inf
+# -x mod -Inf = x
 multimethod mod => qw($ Math::BigNum::Inf) => sub {
-    Math::BigNum->new($_[0]);
+    my ($x, $y) = @_;
+    ($x <=> 0) == Math::GMPq::Rmpq_sgn($$y) ? Math::BigNum->new($x) : $y->copy;
 };
 
 multimethod mod => qw(Math::BigNum::Inf *) => \&nan;
